@@ -21,12 +21,19 @@
 Define some mock classes for the unittests.
 '''
 
-from gi import require_version
+from typing import Any
+from typing import List
+from typing import Optional
+from gi import require_version # type: ignore
 require_version('IBus', '1.0')
-from gi.repository import IBus
+from gi.repository import IBus # type: ignore
 
 class MockEngine:
-    def __init__(self, engine_name = '', connection = None, object_path = ''):
+    def __init__(
+            self,
+            engine_name: str = '',
+            connection: Any = None,
+            object_path: str = ''):
         self.mock_auxiliary_text = ''
         self.mock_preedit_text = ''
         self.mock_preedit_text_cursor_pos = 0
@@ -44,16 +51,16 @@ class MockEngine:
         # engine does not try to support surrounding text, i.e.
         # we omit “| IBus.Capabilite.SURROUNDING_TEXT” here.
 
-    def update_auxiliary_text(self, text, visible):
+    def update_auxiliary_text(self, text: IBus.Text, visible: bool) -> None:
         self.mock_auxiliary_text = text.text
 
-    def hide_auxiliary_text(self):
+    def hide_auxiliary_text(self) -> None:
         pass
 
-    def hide_preedit_text(self):
+    def hide_preedit_text(self) -> None:
         pass
 
-    def commit_text(self, text):
+    def commit_text(self, text: IBus.Text) -> None:
         self.mock_committed_text = (
             self.mock_committed_text[
                 :self.mock_committed_text_cursor_pos]
@@ -62,7 +69,7 @@ class MockEngine:
                 self.mock_committed_text_cursor_pos:])
         self.mock_committed_text_cursor_pos += len(text.text)
 
-    def forward_key_event(self, val, code, state):
+    def forward_key_event(self, val: int, code: int, state: int) -> None:
         if (val == IBus.KEY_Left
             and self.mock_committed_text_cursor_pos > 0):
             self.mock_committed_text_cursor_pos -= 1
@@ -77,131 +84,138 @@ class MockEngine:
                 self.mock_committed_text_cursor_pos:])
             self.mock_committed_text_cursor_pos += len(unicode)
 
-    def update_lookup_table(self, table, visible):
+    def update_lookup_table(
+            self, table: IBus.LookupTable, visible: bool) -> None:
         pass
 
-    def update_preedit_text(self, text, cursor_pos, visible):
+    def update_preedit_text(
+            self, text: IBus.Text, cursor_pos: int, visible: bool) -> None:
         self.mock_preedit_text = text.get_text()
         self.mock_preedit_text_cursor_pos = cursor_pos
         self.mock_preedit_text_visible = visible
 
-    def register_properties(self, property_list):
+    def register_properties(self, property_list: List[IBus.Property]) -> None:
         pass
 
-    def update_property(self, property):
+    def update_property(self, property: IBus.Property) -> None:
         pass
 
-    def hide_lookup_table(self):
+    def hide_lookup_table(self) -> None:
+        pass
+
+    def connect(self, signal: str, callback_function: Any) -> None:
         pass
 
 class MockLookupTable:
-    def __init__(self, page_size = 9, cursor_pos = 0, cursor_visible = False, round = True):
+    def __init__(
+            self,
+            page_size: int = 9,
+            cursor_pos: int = 0,
+            cursor_visible: bool = False,
+            round: bool = True):
         self.clear()
         self.mock_page_size = page_size
         self.mock_cursor_pos = cursor_pos
         self.mock_cursor_visible = cursor_visible
         self.cursor_visible = cursor_visible
         self.mock_round = round
-        self.mock_candidates = []
-        self.mock_labels = []
+        self.mock_candidates: List[str] = []
+        self.mock_labels: List[str]  = []
         self.mock_page_number = 0
 
-    def clear(self):
+    def clear(self) -> None:
         self.mock_candidates = []
         self.mock_cursor_pos = 0
 
-    def set_page_size(self, size):
+    def set_page_size(self, size: int) -> None:
         self.mock_page_size = size
 
-    def get_page_size(self):
+    def get_page_size(self) -> int:
         return self.mock_page_size
 
-    def set_round(self, round):
+    def set_round(self, round: bool) -> None:
         self.mock_round = round
 
-    def set_cursor_pos(self, pos):
+    def set_cursor_pos(self, pos: int) -> None:
         self.mock_cursor_pos = pos
 
-    def get_cursor_pos(self):
+    def get_cursor_pos(self) -> int:
         return self.mock_cursor_pos
 
-    def get_cursor_in_page(self):
+    def get_cursor_in_page(self) -> int:
         return (self.mock_cursor_pos
                 - self.mock_page_size * self.mock_page_number)
 
-    def set_cursor_visible(self, visible):
+    def set_cursor_visible(self, visible: bool) -> None:
         self.mock_cursor_visible = visible
         self.cursor_visible = visible
 
-    def cursor_down(self):
+    def cursor_down(self) -> None:
         if len(self.mock_candidates):
             self.mock_cursor_pos += 1
             self.mock_cursor_pos %= len(self.mock_candidates)
 
-    def cursor_up(self):
+    def cursor_up(self) -> None:
         if len(self.mock_candidates):
             if self.mock_cursor_pos > 0:
                 self.mock_cursor_pos -= 1
             else:
                 self.mock_cursor_pos = len(self.mock_candidates) - 1
 
-    def page_down(self):
+    def page_down(self) -> None:
         if len(self.mock_candidates):
             self.mock_page_number += 1
             self.mock_cursor_pos += self.mock_page_size
 
-    def page_up(self):
+    def page_up(self) -> None:
         if len(self.mock_candidates):
             if self.mock_page_number > 0:
                 self.mock_page_number -= 1
                 self.mock_cursor_pos -= self.mock_page_size
 
-    def set_orientation(self, orientation):
+    def set_orientation(self, orientation: int) -> None:
         self.mock_orientation = orientation
 
-    def get_number_of_candidates(self):
+    def get_number_of_candidates(self) -> int:
         return len(self.mock_candidates)
 
-    def append_candidate(self, candidate):
+    def append_candidate(self, candidate: IBus.Text) -> None:
         self.mock_candidates.append(candidate.get_text())
 
-    def get_candidate(self, index):
+    def get_candidate(self, index: int) -> str:
         return self.mock_candidates[index]
 
-    def get_number_of_candidates(self):
-        return len(self.mock_candidates)
-
-    def append_label(self, label):
+    def append_label(self, label: IBus.Text) -> None:
         self.mock_labels.append(label.get_text())
 
 class MockPropList:
-    def __init__(self, *args, **kwargs):
-        self._mock_proplist = []
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        self._mock_proplist: List[IBus.Property] = []
 
-    def append(self, property):
+    def append(self, property: IBus.Property) ->  None:
         self._mock_proplist.append(property)
 
-    def get(self, index):
+    def get(self, index: int) -> Optional[IBus.Property]:
         if index >= 0 and index < len(self._mock_proplist):
             return self._mock_proplist[index]
         else:
             return None
 
-    def update_property(self, property):
+    def update_property(self, property: IBus.Property) -> None:
         pass
 
 class MockProperty:
     def __init__(self,
-                 key='',
-                 prop_type=IBus.PropType.RADIO,
-                 label=IBus.Text.new_from_string(''),
-                 symbol=IBus.Text.new_from_string(''),
-                 icon='',
-                 tooltip=IBus.Text.new_from_string(''),
-                 sensitive=True,
-                 visible=True,
-                 state=IBus.PropState.UNCHECKED,
-                 sub_props=None):
+                 key: str = '',
+                 prop_type: IBus.PropType = IBus.PropType.RADIO,
+                 label: IBus.Text = IBus.Text.new_from_string(''),
+                 symbol: IBus.Text = IBus.Text.new_from_string(''),
+                 icon: str = '',
+                 tooltip: IBus.Text = IBus.Text.new_from_string(''),
+                 sensitive: bool = True,
+                 visible: bool = True,
+                 state: IBus.PropState = IBus.PropState.UNCHECKED,
+                 sub_props: Optional[List[IBus.Property]] = None) -> None:
         self.mock_property_key = key
         self.mock_property_prop_type = prop_type
         self.mock_property_label = label.get_text()
@@ -213,29 +227,29 @@ class MockProperty:
         self.mock_property_state = state
         self.mock_property_sub_props = sub_props
 
-    def set_label(self, ibus_text):
+    def set_label(self, ibus_text: IBus.Text) -> None:
         self.mock_property_label = ibus_text.get_text()
 
-    def set_symbol(self, ibus_text):
+    def set_symbol(self, ibus_text: IBus.Text) -> None:
         self.mock_property_symbol = ibus_text.get_text()
 
-    def set_tooltip(self, ibus_text):
+    def set_tooltip(self, ibus_text: IBus.Text) -> None:
         self.mock_property_tooltip = ibus_text.get_text()
 
-    def set_icon(self, icon_path):
+    def set_icon(self, icon_path: str) -> None:
         self.mock_property_icon = icon_path
 
-    def set_sensitive(self, sensitive):
+    def set_sensitive(self, sensitive: bool) -> None:
         self.mock_property_sensitive = sensitive
 
-    def set_visible(self, visible):
+    def set_visible(self, visible: bool) -> None:
         self.mock_property_visible = visible
 
-    def set_state(self, state):
+    def set_state(self, state: IBus.PropState) -> None:
         self.mock_property_state = state
 
-    def set_sub_props(self, proplist):
+    def set_sub_props(self, proplist: List[IBus.Property]) -> None:
         self.mock_property_sub_props = proplist
 
-    def get_key(self):
+    def get_key(self) -> str:
         return self.mock_property_key
